<?php

namespace App\Domains\Orders\Delivery\Correios;

use Config, Exception;
use App\Domains\Orders\Exceptions\PostalCodeInvalidException;
use EscapeWork\Frete\Collection;
use Illuminate\Support\Facades\Log;

abstract class BaseCorreios
{
    /**
     * Comprimento minímo e máximo
     */
    const MIN_DEPTH = 16;
    const MAX_DEPTH = 105;

    /**
     * Largura mínima e máxima
     */
    const MIN_WIDTH = 11;
    const MAX_WIDTH = 105;

    /**
     * Min and max Heinght
     */
    const MIN_HEIGHT = 2;
    const MAX_HEIGHT = 105;

    /**
     * Max weight
     */
    const MAX_WEIGHT = 30;

    /**
     * Soma total dos valores
     */
    const MAX_SUM = 200;

    /**
     * Fator multiplicação
     * - Se as medidas finais passarem dos limites, é adicionado o fator multiplicação
     * - Ou seja, o valor do frete vai ser multiplicado pela quantidade de 'pacotes' que será necessária
     */
    protected $multiply = 1;

    public function validate()
    {
        # largura não pode ser maior que 105cm
        if ($this->measures['width'] > self::MAX_WIDTH) {
            $this->measures['width'] = self::MAX_WIDTH;
        }

        # altura não pode ser maior que 105cm
        if ($this->measures['height'] > self::MAX_HEIGHT) {
            $this->measures['height'] = self::MAX_HEIGHT;
        }

        # comprimento
        if ($this->measures['depth'] > self::MAX_DEPTH) {
            $this->measures['depth'] = self::MAX_DEPTH;
        }

        # O comprimento não pode ser inferior a 16 cm.
        if ($this->measures['depth'] < self::MIN_DEPTH) {
            $this->measures['depth'] = self::MIN_DEPTH;
        }

        # A largura não pode ser inferior a 5 cm.
        if ($this->measures['width'] < self::MIN_WIDTH) {
            $this->measures['width'] = self::MIN_WIDTH;
        }

        # A altura não pode ser inferior a 2 cm.
        if ($this->measures['height'] < self::MIN_HEIGHT) {
            $this->measures['height'] = self::MIN_HEIGHT;
        }

        # o peso não pode ser maior que 30kg
        if ($this->measures['weight'] > self::MAX_WEIGHT) {
            return false;
        }

        # A soma do comprimento + largura + altura não deve superar a 200cm.
        if (($this->measures['width'] + $this->measures['height'] + $this->measures['depth']) > self::MAX_SUM) {
            return false;
        }

        # verificando a comparação entre peso x peso cúbico
        if ($this->measures['volume'] > 10) {
            $this->measures['weight'] = $this->measures['volume'];
        }

        return true;
    }

    protected function setDefaultAttributes($cepOrigem, $cepDestino)
    {
        $this->correios->setCepOrigem($cepOrigem)
                       ->setCepDestino($cepDestino)
                       ->setPeso($this->measures['weight'])
                       ->setLargura(number_format(ceil($this->measures['width']), 2, ',', '.'))
                       ->setAltura(number_format(ceil($this->measures['height']), 2, ',', '.'))
                       ->setComprimento(number_format(ceil($this->measures['depth']), 2, ',', '.'))
                       ->setDiametro(0)
                       ->setCodigoEmpresa(configuration('correios_codigo') ?: config('kernel::general.correios.codigo'))
                       ->setSenha(configuration('correios_senha') ?: config('kernel::general.correios.senha'));
    }

    protected function formatValue($value)
    {
        return (float) str_replace(',', '.', str_replace('.', '', $value));
    }

    protected function format($result)
    {
        if ($result instanceof Collection) {
            $result = $result->getFirstResultWithoutError();
        }

        return [
            'price'    => $this->formatValue($result->getValor()) * $this->multiply,
            'deadline' => $result->getPrazoEntrega(),
        ];
    }

    protected function applyMultiplyFactor()
    {
        $multiply = 1;

        if (($width = ceil($this->measures['width'] / self::MAX_WIDTH)) > $multiply) {
            $multiply = $width;
        }

        if (($height = ceil($this->measures['height'] / self::MAX_HEIGHT)) > $multiply) {
            $multiply = $height;
        }

        if (($depth = ceil($this->measures['depth'] / self::MAX_DEPTH)) > $multiply) {
            $multiply = $depth;
        }

        if (($weight = ceil($this->measures['weight'] / self::MAX_WEIGHT)) > $multiply) {
            $multiply = $weight;
        }

        # ajustando as medidas
        $this->measures['width']  = (self::MAX_SUM - 2) / 3;
        $this->measures['height'] = (self::MAX_SUM - 2) / 3;
        $this->measures['depth']  = (self::MAX_SUM - 2) / 3;

        # setando o peso maximo (apenas se ele for maior)
        if ($this->measures['weight'] > self::MAX_WEIGHT) {
            $this->measures['weight'] = self::MAX_WEIGHT;
        }

        # setando o fator multiplicação
        $this->multiply = $multiply;
    }

    public function handleException(Exception $e)
    {
        switch ($e->getErrorCode()) {
            # cep do cliente inválido
            case '-3':
                throw new PostalCodeInvalidException;

            # erro na peso, largura, altura, comprimento ou soma deles
            case '-4': case '-12': case '-13':
            case '-14': case '-15': case '-16':
            case '-17': case '-18': case '-20':
            case '-22': case '-23': case '-24':
            case '-25': case '-26': case '-27':
            case '-28': case '-29': case '-30':
            case '-31': case '-32':
                Log::error('
                    Nenhuma forma de entrega encontrada para o CEP
                    Peso ' . $this->measures['weight']. '
                    Largura ' . $this->measures['width'] . '
                    Altura ' . $this->measures['height'] . '
                    Comprimento ' . $this->measures['depth']
                );
                break;

            default:
                Log::error($e);
        }

        return [
            'error'    => 1,
            'errorMsg' => 'Houve um erro ao calcular o frete'
        ];
    }
}
