<?php

namespace App\Domains\Orders\Delivery\Repositories;

use App\Domains\Orders\Delivery\Models\DeliveryType;
use App\Domains\Orders\Delivery\Models\DeliveryTypeRange;
use App\Domains\Orders\Delivery\DeliveryTypeNotFoundException;
use Illuminate\Support\Collection;

use EscapeWork\LaravelSteroids\Repository;

class DeliveryTypeRepository extends Repository
{
    /**
     * @var ELucre\Models\DeliveryTypeRange
     */
    protected $deliveryTypeRange;

    public function __construct(DeliveryType $deliveryType, DeliveryTypeRange $deliveryTypeRange)
    {
        $this->model             = $deliveryType;
        $this->deliveryTypeRange = $deliveryTypeRange;
    }

    public function all()
    {
        return $this->model
                    ->with('value')
                    ->orderBy('order')
                    ->get();
    }

    public function save()
    {
        if (! $this->model->exists) {
            $this->setOrder();
        }

        $this->model->save();

        if( request("not_available") ){
            $this->model->categories()->sync( request("not_available") );
        }else{
            $this->model->categories()->sync([]);
        }

    }

    public function saveRanges($ranges)
    {
        # excluindo todos os ranges
        $this->deliveryTypeRange->where('delivery_type_id', '=', $this->model->id)->delete();

        foreach ((array) $ranges as $range) {
            $deliveryTypeRange = new DeliveryTypeRange(array(
                'uf'                  => $range['uf'],
                'postal_code_initial' => $range['postal_code_initial'],
                'postal_code_final'   => $range['postal_code_final'],
                'price'               => $range['price'],
                'deadline'            => $range['deadline'],
            ));

            if ($deliveryTypeRange->validate()) {
                $this->model->ranges()->save($deliveryTypeRange);
            }
        }
    }

    public function setOrder()
    {
        $order = $this->model->select('order')->orderBy('order', 'desc')->take(1)->first();

        $this->model->order = $order->order;
    }

    public function getAvailableTypes($measures, $price, $cep, $order = null)
    {
        $query = $this->model->active();

        # filtro por preço mínimo
        $query->where(function($query) use($price)
        {
            $query->whereNull('price_min')
                  ->orWhere('price_min', '<=', $price);
        });

        # filtro por preço máximo
        $query->where(function($query) use($price)
        {
            $query->whereNull('price_max')
                  ->orWhere('price_max', '>=', $price);
        });

        # filtro por peso minimo
        $query->where(function($query) use($measures)
        {
            $query->whereNull('weight_min')
                  ->orWhere('weight_min', '<=', $measures['weight']);
        });

        # filtro por peso maximo
        $query->where(function($query) use($measures)
        {
            $query->whereNull('weight_max')
                  ->orWhere('weight_max', '>=', $measures['weight']);
        });

        /*if( $order ){
            $collection = new Collection();
            $orderCategories = $this->getDeliveryProductsCategories( $order );
            $deliveryTypes = $query->orderBy('order', 'asc')->get();
            foreach( $deliveryTypes as $deliveryType ){
                $not_available_categories = $deliveryType->categories->pluck("id");
                if( ! array_intersect( $not_available_categories, $orderCategories ) ){
                    $collection->push( $deliveryType );
                }
            }

            return $collection;
        }*/

        # retornar o resultado
        return $query->orderBy('order', 'asc')->get();
    }

    /*
    Organizando as categorias do pedido
    */
    public function getDeliveryProductsCategories( $cart ){
        $categories = [];

        foreach( $cart->products() as $order_product ){
            $categories[] = $order_product->category_id;
        }

        return $categories;
    }

    /**
     * Buscando um registro do model DeliveryTypeRange atraves de um range especifico
     */
    public function getOptionByRange($deliveryType, $cep, $weight = null)
    {
        $query =
            $this->deliveryTypeRange
                ->select('delivery_type_ranges.*')
                ->where('delivery_type_ranges.delivery_type_id', '=', $deliveryType->id)
                ->where(function ($query) use ($cep) {
                    $query->where(function ($query) use ($cep) {
                        $query->where('delivery_type_ranges.postal_code_initial', '<=', $cep)
                              ->where('delivery_type_ranges.postal_code_final', '>=', $cep);
                    });
                });

        if ($weight) {
            $query->joinDeliveryTypes()
                  ->where(function ($query) use ($weight) {
                    return $query->where('delivery_types.weight_min','<=' , $weight)
                                 ->orWhereNull('delivery_types.weight_min');
                  });
        }

        return $query->take(1)->first();
    }

    /**
     * Reescrevendo o findOrFail da forma de entrega
     */
    public function findOrFail($id)
    {
        if (! $this->model = $this->model->find($id)) {
            throw new DeliveryTypeNotFoundException('Forma de entrega inválida.');
        }

        return $this;
    }

    public function categoriesIds(){
        $categories = $this->model->categories;
        dd( $categories );
    }
}
