<?php

namespace App\Domains\Orders\Models;

use App\Domains\Data\Datable;
use App\Domains\General\ReadableByAPI;
use App\Domains\Orders\Presenters\OrderPresenter;
use App\Domains\Status\Status;
use App\Domains\Stores\Models\Store;
use App\Domains\Users\Models\User;
use App\Domains\Users\Models\UserAddress;
use App\Domains\Payment\Payable;
use App\Domains\Payment\Models\Payment;
use Carbon\Carbon;
use Dalmolin\Historiable\Historiable;
use App\Domains\General\Model;
use EscapeWork\LaravelSteroids\Ordenable;
use EscapeWork\LaravelSteroids\Presentable;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;

class Order extends Model
{
    use Presentable, Ordenable, Datable, Historiable, ReadableByAPI, Payable;

    /**
     * Table
     */
    protected $table = 'orders';

    /**
     * Fillable fields
     */
    protected $fillable = [
        'user_id',
        'code',
        'address_id',
        'store_id',
        'status_id',
        'delivery_type',
        'total',
        'observation',
        'type',
        'status_pagseguro',
        'transaction_code'
    ];

    /**
     * Dates
     */
    protected $dates = [
        'finalized_at',
    ];

    /**
     * Appends
     */
    protected $appends = [
        'created_at_formated',
    ];

    public function getCreatedAtFormatedAttribute()
    {
        return $this->created_at->format('d/m/Y H:i:s');
    }

    /**
     * Presenter
     */
    protected $presenter = OrderPresenter::class;

    public function user()
    {
        return $this->belongsTo(User::class, 'user_id')
          ->withTrashed();
    }

    public function store()
    {
        return $this->belongsTo(Store::class, 'store_id');
    }

    public function address()
    {
        return $this->belongsTo(UserAddress::class, 'address_id')
                    ->withTrashed();
    }

    public function delivery(){
      return $this->hasOne(OrderDelivery::class, 'order_id');
    }

    public function status()
    {
        return $this->belongsTo(Status::class, 'status_id');
    }

    public function items()
    {
        return $this->hasMany(OrderItem::class, 'order_id');
    }

    public function scopeJoinUsers($query)
    {
        return $query->join('users', 'users.id', '=', 'orders.user_id');
    }

    public function scopeJoinDelivery($query)
    {
        return $query->join('order_delivery', 'order_delivery.order_id', '=', 'orders.id');
    }

    public function scopeJoinItems($query)
    {

        return $query->join(DB::raw('(SELECT product_id, order_id, quantity, ( quantity * price) AS priceTotal FROM order_items ) order_items'),
        function($join)
        {
           $join->on('order_items.order_id', 'orders.id');
        });

        //return $query->join('order_items', 'order_items.order_id', '=', 'orders.id');
    }

    public function scopeJoinProducts($query)
    {
        return $query->join('products', 'order_items.product_id', '=', 'products.id');
    }

    public function scopeJoinStatus($query)
    {
        return $query->join('status', 'status.id', '=', 'orders.status_id');
    }

    public function scopeJoinCategories($query)
    {
        return $query->join('categories', 'products.category_id', '=', 'categories.id');
    }

    public function scopeJoinMainCategories($query)
    {
        return $query->leftJoin('categories as mainCategory', 'categories.category_id', '=', 'mainCategory.id');
    }

    public function scopeFinalized($query)
    {
        return $query->where('status_id', '=', Status::CONFIRMED);
    }

    public function scopeCompleted($query)
    {
        return $query->where('status_id', '=', Status::CONFIRMED);
    }

    public function isOwner($user)
    {
        return $this->user_id == $user->id;
    }

    public function scopePagseguro( $query  ){
        return $query->where("orders.type","payment");
    }

    public function reviewed()
    {
        return $this->getData('reviewed');
    }

    public function isPaid()
    {
        return in_array( $this->status_id,[
          Status::WAITING_PAYMENT,
          Status::CONFIRMED,
          Status::SENDED ,
          Status::DELIVERED
        ]);
    }

    public function isConversion()
    {
        return in_array( $this->status_id,[
          Status::CONFIRMED,
          Status::SENDED ,
          Status::DELIVERED
        ]);
    }

    public function toHistoriableArray()
    {
        $origin = Auth::guard('manager')->user() ? Auth::guard('manager')->user()->name : 'Notificação PagSeguro';
        return [
            'origin'       => $origin,
            'user_id'      => $this->user_id,
            'status_id'    => $this->status->id,
            'status_title' => $this->status->title,
            'total'        => $this->total,
            'observation'  => request()->get('transaction_observation')
        ];
    }

    public static function boot()
    {
        parent::boot();

        static::updating(function($order) {
            if ($order->status_id == Status::CONFIRMED && ! $order->finalized_at) {
                $order->finalized_at = Carbon::now();
            }
        });
    }
}
