<?php

namespace App\Domains\Payment\PagSeguro;

use App\Domains\Payment\PaymentException;
use Exception;
use Illuminate\Support\Facades\App;
use Illuminate\Support\Facades\Config;
use Illuminate\Support\Facades\Log;
use PagSeguro\Configuration\Configure;
use PagSeguro\Library;
use PagSeguro\Services\Transactions\Search\Code;
use App\Domains\Orders\Notifications\OrderUpdatedNotification;
use App\Domains\Orders\Notifications\OrderPaidNotification;
use App\Domains\Orders\Notifications\OrderCancelledNotification;
use App\Domains\Orders\Notifications\OrderCreatedNotification;
use App\Domains\Status\Status as StatusOrder;
use App\Domains\Courses\Notifications\EnrollmentOrderCompletedNotification;
use App\Domains\Orders\Mailers\OrderFinalizedAdminMail;
use App\Domains\Orders\Mailers\OrderUpdatedAdminMail;
use App\Domains\Courses\Mailers\OrderUpdatedAdminMail as EnrollmentOrderUpdatedAdminMail;
use App\Domains\Courses\Mailers\OrderFinalizedAdminMail as EnrollmentOrderFinalizedAdminMail;
use Mail;
use App\Domains\Courses\Models\Order as EnrollmentOrder;
use App\Domains\Orders\Models\Order as ProductOrder;
use Carbon\Carbon;
use App\Models\DID;
use App\Services\IssabelService;
use App\Domains\Orders\Mailers\OrderPaidFinanceiroMail;

trait PaymentBase
{
    /**
     * E-mail do PagSeguro
     */
    protected $email;

    /**
     * Token do PagSeguro
     */
    protected $token;

    public function __construct($email, $token)
    {
        $this->email = $email;
        $this->token = $token;

        $this->boot();
    }

    public function boot()
    {
        Library::initialize();

        if (env('TEST_PAGSEGURO') === true) {
            Configure::setEnvironment('sandbox');
            Configure::setAccountCredentials(
                Config::get('payment.pagseguro.sandbox.email'),
                Config::get('payment.pagseguro.sandbox.token')
            );

            Configure::setLog(true, storage_path('logs/pagseguro.log'));
        } else {
            Configure::setEnvironment('production');
            Configure::setAccountCredentials(
               Config::get('payment.pagseguro.production.email'),
               Config::get('payment.pagseguro.production.token')
           );
        }

        Configure::setCharset('UTF-8');
    }

    public function credentials()
    {
        return Configure::getAccountCredentials();
    }

    protected function addBasePaymentInfos($method, $order, $client, $request)
    {

        $method->setReference($order->id);
        $method->setCurrency('BRL');

        $this->mountProductOrderInfos( $method, $order, $client, $request );

        # infos do cliente
        if($client->name)
        {
          $name = $client->name;
        }else{
          $name = $client->company_name;
        }

        $method->setSender()->setName($name);

        if (env('TEST_PAGSEGURO') === 'local') {
            $method->setSender()->setEmail( config('payment.pagseguro.sandbox.testing.email') );
        } else {
            $method->setSender()->setEmail($client->present->email);
        }

        $method->setSender()->setPhone()->withParameters(
            $request->get('credit_card_phone_ddd'),
            str_replace("-","",$request->get('credit_card_phone'))
        );

        $method->setSender()->setDocument()->withParameters(
            'CPF',
            $request->get('credit_card_cpf')
        );

        # se for boleto ou cartão de crédito, precisa setar o hash
        if ($method instanceof \PagSeguro\Domains\Requests\DirectPayment\Boleto || $method instanceof \PagSeguro\Domains\Requests\DirectPayment\CreditCard) {

            $method->setSender()->setHash($request->get('sender_hash'));
            if( $request->ip() == '::1'){
              $method->setSender()->setIp('192.168.1.1');
            }else{
              $method->setSender()->setIp($request->ip());
            }

        }

        return $method;
    }


    public function mountProductOrderInfos( $method, $order, $client, $request ){

        # adicionando os itens

        foreach ($order->items as $item) {

            if($item->price > 0)
            {
              $method->addItems()->withParameters(
                  $item->id,
                  $item->description,
                  $item->quantity,
                  $item->price
              );
            }

        }

        $method->setShipping()->setAddress()->withParameters(
            $order->address->address,
            $order->address->number,
            $order->address->district,
            $order->address->postal_code,
            $order->address->city,
            $order->address->state,
            'BRA',
            $order->address->address_complement
        );


      # desconto
      if ($order->discount) {
          $method->setExtraAmount($order->discount * -1);
      }

    }

    public function transaction($code)
    {
        return Code::search(
            $this->credentials(),
            $code
        );
    }

    public function updateOrderInfos($order, $transaction)
    {
        $order->transaction_code = $transaction->getCode();
        $status  = new Status($transaction->getStatus());

        if ($status->shouldChangeOrderStatus()) {

            $order->status_id = $status->getStatusId();
            $order->status_pagseguro = $status->getStatus();
            $order->save();

            $this->sendOrderEmails( $order );

            if($order->status_id == StatusOrder::PAYMENT_CANCELED)
            {
              $this->removeFromService($order);
            }

        }

    }

    public function removeFromService($order)
    {

      $webservice = new IssabelService();

      // remove rocs
      $rocs = json_decode($order->getData('issabel_rocs'));
      foreach($rocs as $did)
      {
        $webservice->removeDID($did->id);
        $localDID = app(DID::class)->find($did->local_id);
        $localDID->available = 1;
        $localDID->user_id = null;
        $localDID->save();
      }

      // remove 0800
      $n0800 = json_decode($order->getData('issabel_0800'));
      foreach($n0800 as $did)
      {
        $webservice->removeDID($did->id);
        $localDID = app(DID::class)->find($did->local_id);
        $localDID->available = 1;
        $localDID->user_id = null;
        $localDID->save();
      }

      // remove user
      $webservice->removeUser($order->getData('issabel_login_id'));

      // remove company
      $webservice->removeCompany($order->getData('issabel_company_id'));

    }

    public function sendOrderEmails( $order ){
        switch( $order->status_id ){
          case StatusOrder::CONFIRMED:
            $order->user->notify(new OrderPaidNotification($order));
            Mail::to(config('mail.to_financeiro'))->send(new OrderPaidFinanceiroMail($order));
            break;
          case StatusOrder::PAYMENT_CANCELED:
            $order->user->notify(new OrderCancelledNotification($order));
            break;
          default:
            $order->user->notify(new OrderUpdatedNotification($order));
            break;
        }

    }

    public function handleException(Exception $e, $order, $client, $request)
    {

        Log::debug( $e );

        $data = [];

        $errors = simplexml_load_string($e->getMessage());
        foreach ( $errors->error as $error) {

            if (isset($error->code)) {
                $data['errors']['erro_'.$error->code] =[ $this->convertErrorPagSeguro( $error )];
            }
        }

        $exception = new PaymentException();
        $exception->setErrors($data);
        throw $exception;
    }

    public function convertErrorPagSeguro( $error ){
       switch( $error->code ){
          case 53141:
            return 'O e-mail informado está bloqueado no Pagseguro.';
            break;
          case 53018:
       		case 53049:
                 return 'Por favor informe o DDD';
                 break;
       		case 53042:
                 return 'Por favor informe o Nome do Titular';
                 break;
       		case 53047:
                 return 'Por favor informe a Data de Nascimento';
                 break;
       		case 53051:
               case 53020:
                 return (string) 'Por favor informe o telefone';
                 break;
           case 53045:
           case 53118:
             return (string) 'Por favor informe o CPF do títular do Cartão de Crédito';
             break;
           default:
             return (string) 'Erro PagSeguro #' . $error->code . ': '. $error->message;
             break;
         }
     }
}
