<?php

namespace App\Domains\Users\Models;

use Carbon\Carbon;
use App\Domains\Data\Datable;
use App\Domains\General\Model;
use App\Domains\Orders\Models\Order;
use App\Domains\Courses\Models\Order as EnrollmentOrder;
use App\Domains\Wishlist\Models\Wishlist;
use App\Domains\Products\Models\Product;
use Illuminate\Auth\Authenticatable;
use App\Domains\General\ReadableByAPI;
use Illuminate\Notifications\Notifiable;
use App\Domains\Courses\Models\Enrollment;
use App\Domains\Coupons\Models\Coupon;
use EscapeWork\LaravelSteroids\Presentable;
use App\Exceptions\UserMissingDataException;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Auth\Passwords\CanResetPassword;
use Illuminate\Foundation\Auth\Access\Authorizable;
use App\Domains\Users\Validators\UserIsFullyFilledValidator;
use App\Domains\Users\Notifications\ResetPasswordNotification;
use Illuminate\Contracts\Auth\Authenticatable as AuthenticatableContract;
use Illuminate\Contracts\Auth\CanResetPassword as CanResetPasswordContract;

class User extends Model implements AuthenticatableContract, CanResetPasswordContract
{
    use Authenticatable, CanResetPassword, SoftDeletes, Presentable, Notifiable, Authorizable, Datable, ReadableByAPI;

    /**
     * Table
     */
    protected $table = 'users';

    /**
     * Fillable fields
     */
    protected $fillable = [
        'preference',
        'segment',
        'contact_name',
        'contact_email',
        'contact_phone',
        'contact_birthday',
        'cpf',
        'cnpj',
        'type',
        'name',
        'company_name',
        'email',
        'phone',
        'password',
        'postal_code',
        'state',
        'city',
        'city_code',
        'address',
        'address_complement',
        'number',
        'district',
        'neighborhood',
        'document',
        'ie',
        'facebook_id',
        'mail',
        'last_login_at',
        'birthday'
    ];

    /**
     * Dates
     */
    protected $dates = [
        'last_login_at'
    ];

    /**
     * Hidden fields
     */
    protected $hidden = [
        'password',
        'remember_token',
        'facebook_id',
        'deleted_at',
        'last_login_at',
    ];

    /**
     * Presenter
     */
    protected $presenter = 'App\Domains\Users\Presenters\UserPresenter';

    public function orders()
    {
        return $this->hasMany(Order::class, 'user_id');
    }

    public function payments()
    {
        return $this->hasMany(Order::class, 'user_id')->where("type","payment");
    }

    public function paymentsToConversion()
    {
        return $this->hasMany(Order::class, 'user_id')->where("type","payment")->where("created_at",">=","2019-06-12 00:00:00");
    }

    public function enrollmentOrders()
    {
        return $this->hasMany(EnrollmentOrder::class, 'user_id');
    }

    public function wishlist()
    {
        return $this->hasMany(Wishlist::class);
    }

    public function applications()
    {
        return $this->hasMany(Application::class, 'user_id');
    }

    public function coupons()
    {
        return $this->hasMany(Coupon::class, 'client_id');
    }

    public function enrollments()
    {
        return $this->hasMany(Enrollment::class, 'user_id')
                    ->with('course');
    }

    public function activeEnrollments()
    {
        return $this->hasMany(Enrollment::class, 'user_id')
                    ->with('course')
                    ->where('status','=', Enrollment::STATUS_CONFIRMADO);
    }

    public function address()
    {
        return $this->hasOne(UserAddress::class, 'user_id');
    }

    public function addresses()
    {
        return $this->hasMany(UserAddress::class, 'user_id');
    }

    public function updateLastLogin()
    {
        $this->last_login_at = Carbon::now();
        $this->save();
    }

    public function setEmailAttribute($value)
    {
        $this->attributes['email'] = mb_strtolower($value);
    }

    public function setPasswordAttribute($value)
    {
        if ($value) {
            $this->attributes['password'] = bcrypt($value);
        }
    }

    public function setDocumentAttribute($document)
    {
        $this->attributes['document'] = strip_document($document);
    }

    /**
     * @throw App\Exceptions\UserMissingDataException
     */
    public function checkIfIsFullyFilled($options = [])
    {
        $options = array_merge([
            'redirect_url' => route('home'),
            'top_message'  => 'Para continuar, precisamos que você complete os seguintes dados: ',
        ], $options);

        $validator = (new UserIsFullyFilledValidator)->validate($this->toArray());

        if ($validator->fails()) {
            $e = new UserMissingDataException('Existem alguns dados faltando para o usuário ' . $this->name);
            $e->setRedirectUrl($options['redirect_url']);
            $e->setTopMessage($options['top_message']);
            $e->setErrors($validator->messages());

            throw $e;
        }
    }

    public function isPerson()
    {
        return true;
    }

    public static function boot()
    {
        parent::boot();

        static::updated(function($user) {
            $user->markAsUnread();
        });
    }

    public function sendPasswordResetNotification($token)
    {
        $this->notify(new ResetPasswordNotification($token));
    }

    public function setBirthdayAttribute($value)
    {
        return $this->_setDateAttribute('birthday', $value);
    }

    public function getBirthdayAttribute()
    {
        if( $this->attributes['birthday'] ){
            return date('d/m/Y',strtotime($this->attributes['birthday']));
        }

        return "";
    }

}
