<?php

namespace App\Http\Controllers\App;

use App\Domains\Orders\Services\CartService;
use App\Domains\Orders\Services\OrderPaymentService;
use App\Http\Requests\OrderFinalizeRequest;
use App\Http\Requests\StepAddressRequest;
use App\Http\Requests\StepCardRequest;
use App\Http\Requests\StepDataRequest;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use App\Domains\Orders\Models\Order;
use App\Domains\Orders\Repositories\OrderRepository;
use App\Services\IssabelService;
use Illuminate\Support\Str;
use App\Domains\Orders\Notifications\OrderCreatedNotification;
use App\Domains\Orders\Notifications\OrderPaidNotification;
use App\Domains\Orders\Notifications\OrderCencelledNotification;
use App\Domains\Orders\Mailers\OrderCreatedAdminMail;
use App\Domains\Orders\Mailers\OrderUpdatedAdminMail;
use App\Domains\Orders\Mailers\OrderPaidFinanceiroMail;
use Illuminate\Support\Facades\Mail;
use App\Domains\Payment\Exceptions\PaymentException;
use Illuminate\Support\Facades\App;
use App\Domains\Payment\PagSeguro\PaymentBase;
use App\Domains\Payment\PagSeguro\CreditCard\Payment;
use App\Models\DID;
use App\Domains\Status\Status;

class OrdersController extends Controller
{

    /**
  	 * @var App\Repositories\OrderRepository
  	 */
  	protected $orders;

    /*
    * Inicializa o controlador
    * @param object $orders Contem a referencia da classe OrderRepository
    */

  	public function __construct( OrderRepository $orders)
  	{
  			$this->orders = $orders;
        $this->webservice = new IssabelService();
  	}

    /*
    * Ao selecionar o primeiro DID disponível, tenta criar a empresa utilizando o webservice
    * @param object $order Contem as informacoes do pedido do checkout
    * @param object $roc Contem as informacoes de um DID selecionado no checkout
    * @param object $did Contem as informacoes de um DID selecionado do banco de dados
    */

    public function createCompany($order, $roc, $did)
    {

      $did->user_id = $order->user->id;
      $did->available = 0;
      $did->save();

      // criando a empresa
      $maxExtensions = 7;

      if($order->user->name)
      {
        $name = $order->user->name;
      }else{
        $name = $order->user->company_name;
      }

      $data = [
        "name" => $name,
        "maxExtensions" => $maxExtensions,
        "colorSchema" => "cyan-cloud",
        "companyGroup" => null,
        "recordCalls" => 111,
        "context" => "from-users",
        "sipUser" => $did->area_code.$did->number,
        "sipPassword" => $did->area_code.$did->number,
      ];

      $company = $this->webservice->createCompany($data);
      $companyData = $company->data;

      // salvando a referencia da empresa no banco de dados
      $order->saveData('issabel_company_id', $companyData->id);

      return $companyData;
    }

    /*
    * No final de todas as etapas do carrinho, finaliza os processos de criação de DIDs, Empresa e Login no webservice.
    * Caso aconteça algum erro, executa roolback no banco e remove os dados criados no webservice.

    * @param object $request recebe uma variavel da aplicacao com os dados enviados pelo usuario
    * @param object $cart recebe uma instancia da classe CartService
    * @param object $creator recebe uma instancia da classe OrderPaymentService
    */

    public function store(Request $request, CartService $cart, OrderPaymentService $creator)
    {

      DB::beginTransaction();
      try {

        /*
        Enviando dados da ordem e pagamento para o PagSeguro
        */

        $order = $creator->handle($request);

        if( !$order )
        {
            DB::rollBack();
            $exception = new PaymentException();
            $exception->setErrors(['errors' => ['error_generic' => 'Não foi possível concluir a sua transação. Por favor tente novamente']]);
            throw $exception;
        }else{
            $this->createSession($request, $order );
        }

        /*
        Os dados para a criacao do pedido foram informados corretamente.
        Criando itens no webservice.
        */

        $companyData = null;
        $companyCreated = false;

        /*
        Para cada numero ROC/POP selecionado no carrinho, reserva o numero no webservice.
        */

        if($request->has('roc_numbers'))
        {
           $rocs = (object) $request->get('roc_numbers');
           $listRocs = [];

           foreach($rocs as $roc)
           {
             $roc = (object) $roc;
             if($roc->quantity > 0)
             {
               for($i=0;$i<$roc->quantity;$i++)
               {

                 /*
                 Seleciona o próximo número DID que ja havia sido reservado no banco de dados
                 */

                 $did = $this->webservice->getAvailableDID('roc', null, $roc->city);
                 $did->user_id = $order->user->id;
                 $did->available = 0;
                 $did->save();

                 if($companyCreated == false)
                 {

                   /*
                  Ao selecionar o primeiro numero, cria a empresa no webservice
                   */

                  $companyData = $this->createCompany($order, $roc, $did);
                  $companyCreated = true;
                 }

                 //create DID
                 $didData = [
                   "number" => $did->area_code.$did->number,
                   "company" => $companyData->id,
                   "custom" => true,
                   "description" => $did->area_code.$did->number
                 ];

                 $didCreated = $this->webservice->createDid($didData);
                 $didCreatedData = $didCreated->data;

                 $listRocs[] = (object) ['id' => $didCreatedData->id, 'local_id' => $did->id, 'city' => $did->city, 'area_code' => $did->area_code, 'number' => $did->number];

               }

             }
           }

           $order->saveData('issabel_rocs', json_encode($listRocs));
        }

        /*
        Se foram selecionados numeros 0800, seleciona e cria os DIDs no webservice
        */

        if($request->has('plan_0800'))
        {
         $plan_0800 = (object) $request->get('plan_0800');

         $list0800 = [];

         if($plan_0800->quantity > 0)
         {
           $plan = (object) $plan_0800->plan;

           for($i=0;$i<$plan_0800->quantity;$i++)
           {
             $did = $this->webservice->getAvailableDID('0800');
             $did->user_id = $order->user->id;
             $did->available = 0;
             $did->save();

             //create DID
             $didData = [
               "number" => $did->area_code.$did->number,
               "company" => $companyData->id,
               "custom" => true,
               "description" => $did->area_code.$did->number
             ];

             $didCreated = $this->webservice->createDid($didData);
             $didCreatedData = $didCreated->data;

             $list0800[] = (object) ['id' => $didCreatedData->id, 'local_id' => $did->id,   'city' => $did->city, 'area_code' => $did->area_code, 'number' => $did->number];

           }

         }

         $order->saveData('issabel_0800', json_encode($list0800));

       }

       /*
       Cria o login no webservice
       */

        $password = Str::random(12);
        $data = [
          "email" => $order->user->email,
          "password" =>  $password,
          "company" =>  $companyData->id,
          "fullName" =>  $order->user->name ? $order->user->name : $order->user->company_name,
          "role" =>  "admin"
        ];

        $user = $this->webservice->createUser($data);
        $userData = $user->data;

        $order->saveData('issabel_login_id', $userData->id);
        $order->saveData('issabel_email', $order->user->email);
        $order->saveData('issabel_password', $password);

        /*
        Se tudo for criado corretamente no site e no webservice,
        sera enviado para o PagSeguro a tentativa de cobranca
        */

        $order = $this->executePayment($order, $order->user, $request);

        DB::commit();

        /*
        Enviar emails transacionais pra quem é necessário
        */

        Mail::to(config('mail.to'))->send(new OrderCreatedAdminMail($order));
        $order->user->notify(new OrderCreatedNotification($order));

        if($order->status_id == 2)
        {
          $order->user->notify(new OrderPaidNotification($order));
          Mail::to(config('mail.to'))->send(new OrderUpdatedAdminMail($order));
          Mail::to(config('mail.to_financeiro'))->send(new OrderPaidFinanceiroMail($order));
        }

        if($order->status_id == 5)
        {
          $order->user->notify(new OrderCencelledNotification($order));
          Mail::to(config('mail.to'))->send(new OrderUpdatedAdminMail($order));
        }

        return response()->json([
            'order' => $order->toArray(),
            'url' => route("orders.finalized", $order->code )
        ]);

      }catch (PaymentException $exception) {

          /*
          Se acontecer algum erro no pagamento, ira remover os dados do webservice e do banco de dados
          */

          $this->removeFromService($order);
          DB::rollBack();

          return response()->json( $e->getErrors() , 422);

      } catch (\Exception $e) {

          /*
          Se acontecer outros erros de integracao com o webservice, ira remover os dados do webservice e do banco de dados
          */

          $this->removeFromService($order);

          DB::rollBack();

          if( method_exists( $e, 'getErrors') )
          {
            return response()->json( $e->getErrors() , 422);
          }else{
            return response()->json( ['errors' => ['generic' => ['Ocorreu um erro ao finalizar o seu pedido. Por favor tente novamente.'] ]] , 422);
          }

      }
    }

  /*
  Remove os dados criados no webservice durante a execucao da ordem , caso aconteca algum erro
  * @param object $order recebe uma instancia com as informacoes do pedido atual
  */

  public function removeFromService($order)
  {
    $webservice = new IssabelService();

    // remove rocs
    $rocs = $order->getData('issabel_rocs', null);
    if($rocs)
    {
      $rocs = json_decode($order->getData('issabel_rocs'));
      foreach($rocs as $did)
      {
        $webservice->removeDID($did->id);
        $localDID = app(DID::class)->find($did->local_id);
        if($localDID)
        {
          $localDID->available = 1;
          $localDID->user_id = null;
          $localDID->save();
        }

      }
    }

    // remove 0800
    $n0800 = $order->getData('issabel_0800', null);
    if($n0800)
    {
      $n0800 = json_decode($order->getData('issabel_0800',[]));
      foreach($n0800 as $did)
      {
        $webservice->removeDID($did->id);
        $localDID = app(DID::class)->find($did->local_id);
        if($localDID)
        {
          $localDID->available = 1;
          $localDID->user_id = null;
          $localDID->save();
        }
      }
    }

    // remove user
    $login = $order->getData('issabel_login_id', null);
    if($login)
    {
      $webservice->removeUser($order->getData('issabel_login_id'));
    }

    // remove company
    $company = $order->getData('issabel_company_id', null);
    if($company)
    {
      $webservice->removeCompany($order->getData('issabel_company_id'));
    }

  }

  /*
  * Executa o pagamento utilizando PagSeguro
  * Para testes, deixar o .env configurado com TEST_PAGSEGURO=true
  * @param object $order recebe uma instancia com as informacoes do pedido atual
  * @param object $user recebe uma instancia com as informacoes do usuario relacionado ao pedido atual
  * @param object $request recebe uma variavel da aplicacao com os dados enviados pelo usuario
  */

  public function executePayment($order, $user, $request)
  {

      if (App::environment() === 'local') {
            $payment = new Payment(
            config('payment.pagseguro.sandbox.testing.email'),
            config('payment.pagseguro.sandbox.token')
        );
      }else{
        $payment = new Payment(
            config('payment.email'),
            config('payment.token')
        );
      }

      $response = $payment->execute($order, $user, $request);

      if ( $response->getStatus() != 1) {
          DB::rollBack();

          $exception = new PaymentException();
          $exception->setErrors(['errors' => ['error_generic' => 'Não foi possível concluir a sua transação. Por favor tente novamente']]);
          throw $exception;
      };

      $address = (object) $request->get('modal');
      $plan = (object) $request->get('plan');

      $data = (object) $request->all();
      $credit_cart = (object) $data->creditCardBrand;

      $installments = $data->plots;
      $plot = (object) $installments[ $request->get('installment') ];

      /*
      Executa o pagamento utilizando a API do PagSeguro e configura como status 1 = Aguardando Pagamento.
      */

      $order->createPayment([
          'type'         => 'pagseguro',
          'code'         => $response->getCode(),
          'status'       => 'Aguardando Pagamento',
          'price'        => $plot->totalAmount,
          'installments' => $response->getInstallmentCount(),
          'card_type'    => $credit_cart->name,
          'card_first6'  => $data->credit_card_number,
          'extra'        => [
              "NumPedido"  => $order->id ,
              "plots"  => [
                  "quantity" => $response->getInstallmentCount(),
                  "installmentAmount"  => $plot->installmentAmount,
                  "grossAmount" => $response->getGrossAmount(),
                  "netAmount"  => $response->getNetAmount(),
                  "feeAmount" => $response->getFeeAmount()
              ]
          ]
      ]);

      $order->status_id = Status::WAITING_PAYMENT;
      $order->transaction_code = $response->getCode();
      $order->save();

      return $order;
  }

  public function storeDelivery( Request $request, OrderPaymentService $creator){

    DB::beginTransaction();

    try {
      $order = $creator->handleOnDelivery($request);
      DB::commit();

      $this->createSession($request, $order );

      return response()->json([
          'order' => $order->toArray(),
          'url' => route("orders.finalized" )
      ]);


    } catch (\Exception $e) {
        DB::rollBack();
        if( method_exists( $e, 'getErrors') ){
          return response()->json( $e->getErrors() , 422);
        }else{
          return response()->json( ['errors' => ['generic' => $e->getMessage() ]] , 422);
        }

    }
  }

  /*
  *Verifica se todos os parametros necessarios para a criacao da ordem no PagSeguro foram preenchidos no carrinho
  * Utiliza o validador App\Http\Requests\OrderFinalizeRequest

  * @param object $request recebe a referencia da class App\Http\Requests\OrderFinalizeRequest
  */

  public function check(OrderFinalizeRequest $request)
  {
    return response()->json([
        'status' => 200,
        'message' => 'Pedido pode ser enviado para o Pagseguro'
    ]);
  }

  /*
  * Exibe a tela de finalizacao, com os dados do pedido

  * @param string $code recebe uma string com o codigo do pedido
  */

  public function finalized($code = null){

    $order = $this->orders->findByCode( $code );
    $user = $order->user;

    if( !$order ){
      return redirect()->route("home");
    }

    return view('app.order', [
        'page'   => 'order',
        'order' => $order,
    ]);

  }

  /*
  * Cria o ID de sessao
  * Esse ID utilizado para vincular os DIDs selecionados no carrinho no banco de addos
  * @param object $request recebe uma variavel da aplicacao com os dados enviados pelo usuario
  * @param object $order recebe uma instancia com as informacoes do pedido atual
  */

  public function createSession( $request, $order ){
      $request->session()->put('transaction_code', $order->transaction_code );
  }

  public function step_data(StepDataRequest $request)
  {
    // validacao acontece no App\Http\Requests\StepDataRequest
  }

  public function step_address(StepAddressRequest $request)
  {
    // validacao acontece no App\Http\Requests\StepAddressRequest
  }

  public function step_card(StepCardRequest $request)
  {
    // validacao acontece no App\Http\Requests\StepCardRequest
  }

}
