<?php

namespace App\Models;

use Carbon\Carbon;
use EscapeWork\LaraMedias\Contracts\Mediable;
use EscapeWork\LaraMedias\Traits\Medias;
use EscapeWork\LaravelSteroids\Cacheable;
use EscapeWork\Manager\Models\Model;
use EscapeWork\LaravelSteroids\Presentable;
use EscapeWork\LaravelSteroids\Sluggable;
use EscapeWork\LaravelSteroids\SortableScope;

class News extends Model implements Mediable
{
    use Presentable, Sluggable, Cacheable, Medias;

    /**
     * Table
     */
    protected $table = 'news';

    /**
     * Fillable fields
     */
    protected $fillable = [
        'news_category_id',
        'title',
        'subtitle',
        'date',
        'content',
        'active',
    ];

    /**
     * Dates
     */
    protected $dates = ['date'];

    /**
     * Caches
     */
    protected $cacheables = [
        'news.latest',
    ];

    /**
     * The presenter class
     */
    protected $presenter = 'App\Presenters\NewsPresenter';

    public function scopeActive($query)
    {
        return $query->where('active', '=', 1);
    }

    public function scopePublished($query)
    {
        return $query->where('date', '<=', Carbon::now()->format('Y-m-d'));
    }

    public function setDateAttribute($date)
    {
        return $this->_setDateAttribute('date', $date);
    }

    public function medias()
    {
        return $this->morphMany('EscapeWork\LaraMedias\Models\Media', 'medias', 'media_model', 'model_id')
                    ->orderBy('order');
    }

    public function related()
    {
        return $this->hasMany('App\Models\News', 'active', 'active')
                    ->with('medias')
                    ->where('id', '<>', $this->id)
                    ->active()
                    ->take(2);
    }

    public function category()
    {
        return $this->belongsTo('App\Models\NewsCategory', 'news_category_id');
    }

    public static function boot()
    {
        parent::boot();
        static::addGlobalScope(new SortableScope('date', 'desc'));
    }

    public function scopeSearch($query, $field, $value)
    {
      return $query->where($field, 'LIKE', '%' . $value . '%');
    }

    public function uploadPicture($picture)
    {
        if ($picture) {
            $this->uploadSingleMedia($picture, 'picture');
            $this->save();
        }
    }

    public function uploadBlob($picture, $field)
    {
        if ($picture) {

            $image = base64_decode(str_replace('data:image/jpeg;base64,','',$picture));
            $safeName = str_random(15).'.'.'jpg';
             \Storage::disk('local')->put('medias/'.$this->table.'/'.$safeName, $image);
             $this->$field = $safeName;
             $this->save();
        }
    }
}
