<?php

namespace App\Services;

use App\Models\Product;
use App\Models\Subscriber;
use DrewM\MailChimp\MailChimp;
use App\Repositories\InfoRepository;
use App\Repositories\DIDRepository;
use Carbon\Carbon;
use Log;

class IssabelService
{
    /**
     * @var DrewM\MailChimp\MailChimp
     */
    protected $url;

    public function __construct()
    {
        $this->url = config('services.issabel.url');
    }

    /*
    Solicita um token de autenticacao para o webservice Issabel
    */

    public function getToken()
    {
      $info = app(InfoRepository::class)->where('key','api_token')->first();

      $expireDate = new Carbon($info->extra);
      $nowDate = Carbon::now();

      // create new token
      $response = $this->doPOST($this->url . 'login', [
         "email" => config('services.issabel.login'),
	       "password" => config('services.issabel.password')
      ]);


      try{
        $token = $response->access_token;
        $expire_date = Carbon::now()->addSeconds($response->expires_in);
        $info->value = $token;
        $info->extra = $expire_date;
        $info->save();

        return $info->value;

      }catch(\Exepction $e){

        /*
        Caso um erro aconteça, exibe erro na tela.
        */

        dd($e);
      }

    }

    /*
    Verifica o proximo numero disponivel para atribuicao no checkout
    */

    public function getAvailableDID($type, $area_code = null, $city = null)
    {
      return app(DIDRepository::class)->findNext($type, $area_code, $city);
    }

    /*
    Cria a empresa no webservice
    */

    public function createCompany($data)
    {

      $sipData = [
  			"sipServer" => config('services.issabel.sip_domain'),
  			"sipDomain" => config('services.issabel.sip_domain'),
  			"fallback" => false
      ];

      $newData = array_merge($data, $sipData);

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doPOST($this->url . 'company', $newData, $headers);

    }

    /*
    Quando ha erro, essa funcao remove os dados de uma empresa previamente criada no webservice
    */

    public function removeCompany($id)
    {

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doDELETE($this->url . 'company/'.$id, [], $headers);

    }

    /*
    Quando ha erro, essa funcao remove os dados de usuario previamente criado no webservice
    */

    public function removeUser($id)
    {

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doDELETE($this->url . 'user/'.$id, [], $headers);

    }

    /*
    Quando ha erro, essa funcao remove os dados de DID previamente criado no webservice
    */

    public function removeDID($id)
    {

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doDELETE($this->url . 'did/'.$id, [], $headers);

    }

    /*
    Cria um novo usuario no webservice
    */

    public function createUser($data)
    {

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doPOST($this->url . 'user', $data, $headers);

    }

    /*
    Cria e atribui um DID no webservice
    */

    public function createDID($data)
    {

      $headers = [
        'Authorization: Bearer ' . $this->getToken()
      ];

      return $this->doPOST($this->url . 'did', $data, $headers);

    }

    /*
    Funcao de execucao dos comandos enviados ao Issabel via metodo POST
    */

    public function doPOST($url, $data, $headers = null)
    {

      $post_fields = http_build_query($data);

      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_TIMEOUT, 30);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
      curl_setopt($ch, CURLOPT_POST, true);
      curl_setopt($ch, CURLOPT_POSTFIELDS, $post_fields);

      if($headers)
      {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
      }

      $responseContent = curl_exec($ch);

      try{
        return json_decode($responseContent);
      }catch(\Exception $e)
      {
        //Log::debug($responseContent);
        return $e;
      }

    }

    /*
    Funcao de execucao dos comandos enviados ao Issabel via metodo DELETE
    */

    public function doDELETE($url, $data, $headers = null)
    {

      $post_fields = http_build_query($data);

      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_TIMEOUT, 30);
      curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
      curl_setopt($ch, CURLOPT_CUSTOMREQUEST, 'DELETE');

      if($headers)
      {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
      }

      $responseContent = curl_exec($ch);

      \Log::debug($responseContent);

      try{
        return json_decode($responseContent);
      }catch(\Exception $e)
      {

        return $e;
      }

    }

    /*
    Funcao de execucao dos comandos enviados ao Issabel via metodo GET
    */

    public function doGET($url, $headers = null)
    {
      $ch = curl_init();
      curl_setopt($ch, CURLOPT_URL, $url);
      curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
      curl_setopt($ch, CURLOPT_TIMEOUT, 30);
      curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

      if($headers)
      {
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
      }

      $responseContent     = curl_exec($ch);

      return json_decode($responseContent);
    }

    public function __call($method, $parameters)
    {
        return call_user_func_array([$this->client, $method], $parameters);
    }
}
